---
eip: 2831
title: Transaction Replacement Message Type
author: Gregory Markou (@GregTheGreek)
discussions-to: https://ethereum-magicians.org/t/eip-2831-transaction-replacement-message-type/4448
status: Stagnant
type: Standards Track
category: Interface
created: 2020-07-26
requires: 1193
---

## Summary

An extension to the JavaScript Ethereum Provider API ([EIP-1193](./eip-1193.md)) this creates a new message type in the event a transaction replacement occurs.

## Abstract

The current communication between providers and consumers of providers are fundamentally broken in the event that a transaction in the mempool has been superseded by a newer transactions. Providers currently have no way of communicating a transaction replacement, and consumers are required to poll block by block for the resulting transaction.

## Motivation

Exert from EIP-1193
> A common convention in the Ethereum web application ("dapp") ecosystem is for key management software ("wallets") to expose their API via a JavaScript object in the web page.
This object is called "the Provider".

Many ingenious developments have been made by wallet developers to improve the overall user experience while interacting with the Ethereum blockchain. One specific innovation was transaction replacement, offering users the ability to effectively cancel a previously sent transaction.

Transaction replacement is not a new concept, but unfortunately causes major user experience problems for dapp developers as the replaced transaction is near impossible to track. 

This EIP formalizes a way for both providers and dapp developers to track transaction replacements seamlessly.

## Specification

The key words "MUST", "MUST NOT", "REQUIRED", "SHALL", "SHALL NOT", "SHOULD", "SHOULD NOT", "RECOMMENDED", "MAY", and "OPTIONAL" in this document are to be interpreted as described in [RFC-2119](https://www.ietf.org/rfc/rfc2119.txt).

### Definitions

_This section is non-normative._

- Provider
  - A JavaScript object made available to a consumer, that provides access to Ethereum by means of a Client.
- Wallet
  - An end-user application that manages private keys, performs signing operations, and acts as a middleware between the Provider and the Client.
- Transaction Replacement
  - Submitting a transaction with both: the same nonce and a 10% increase in the gas price, of a previous transaction which a user no longer wishes to send. This must occur before the original transaction is included in the blockchain.

### Events

These methods **MUST** be implemented per the Node.js [`EventEmitter` API](https://nodejs.org/api/events.html).

The following three events must be implemented: `tx_replacement`, `tx_speedup` and `tx_cancel`.

A `tx_speedup` is defined as a transaction replacement in which the user wishes to adjust the `gasPrice`, to potentially receive a fast block inclusion. For a `tx_speedup` to be considered valid, the replacement tx must contain the **same** following properties as the one it supersedes:
- Nonce
- To
- Value
- Data

```typescript
interface txSpeedupInfo {
  readonly oldTx: string;
  readonly newTx: string;
  readonly nonce: string;
  readonly from: string;
}

Provider.on('tx_speedup', listener: (txSpeedupInfo: txSpeedupInfo) => void): Provider;
```
This event emits the old transaction hash (`oldTx`), the new transaction hash (`newTx`), the nonce used for both transactions (`nonce`), and the signing address for the transaction (`from`).

A `tx_cancel` is defined as a transaction replacement in which the user wishes to nullify a previous transaction before its inclusion. For a `tx_cancel` to be considered valid, the replacement tx must contain the following properties:
- The same nonce as the superseded transaction
- The same From and To
- Zero value
- No data

```typescript
interface txCancelInfo {
  readonly oldTx: string;
  readonly newTx: string;
  readonly nonce: string;
  readonly from: string;
}

Provider.on('tx_cancel', listener: (txCancelInfo: txCancelInfo) => void): Provider;
```
This event emits the old transaction hash (`oldTx`), the new transaction hash (`newTx`), the nonce used for both transactions (`nonce`), and the signing address for the transaction (`from`).

A `tx_replacement` is defined as a transaction replacement in which a user has completely replaced a previous transaction with a completely brand new one. The replacement tx must contain the following properties:
- The same nonce as the superseded transaction

```typescript
interface txReplacementInfo {
  readonly oldTx: string;
  readonly newTx: string;
  readonly nonce: string;
  readonly from: string;
}

Provider.on('tx_replacement', listener: (txReplacementInfo: txReplacementInfo) => void): Provider;
```
This event emits the old transaction hash (`oldTx`), the new transaction hash (`newTx`), the nonce used for both transactions (`nonce`), and the signing address for the transaction (`from`).

## Rationale

The implementation was chosen to help the ease of implementation for both providers and dapp developers. Since `ProviderMessage` is widely used by dapp developers already it means that the implementation path would be as trivial as adding and additional `if` clause to their existing message listener. This also provides a benefit to dapps in the event that a provider has not yet implemented the events, it will not cause the dapp panic with `undefined` should it be implemented natively (eg: `ethereum.txCancel(...)` which would error with `ethereum.txReplacement()` is not a function).

## Backwards Compatibility

Many Providers adopted EIP-1193, as this EIP extends the same event logic, there should be no breaking changes. All providers that do not support the new events should either I) Ignore the subscription or II) Provide some error to the user.

## Implementations

- [Web3.js](https://github.com/ethereum/web3.js/issues/3723)
- [MetaMask](https://github.com/MetaMask/metamask-extension/issues/9174)

## Security Considerations

None at the current time.

## References

- [Web3.js issue with metamask tx cancel](https://github.com/ethereum/web3.js/issues/3585)
- [Browser doesn't know when a transaction is replace](https://github.com/MetaMask/metamask-extension/issues/3347)

## Copyright

Copyright and related rights waived via [CC0](../LICENSE.md).

## Appendix I: Examples

These examples assume a web browser environment.

```javascript
// Most Providers are available as window.ethereum on page load.
// This is only a convention, not a standard, and may not be the case in practice.
// Please consult the Provider implementation's documentation.
const ethereum = window.ethereum;

const transactionParameters = { ... } // Fill in parameters

ethereum
  .request({ 
    method: 'eth_sendTransaction',
    params: [transactionParameters],
  })
  .then((txHash) => {
    ethereum.on('tx_cancel', (info) => {
      const { oldTx, newTx, nonce, from } = message.data;
      console.log(`Tx ${oldTx} with nonce ${nonce} from ${from} was cancelled, the new hash is ${newTx}`)
    });
    ethereum.on('tx_speedup', (info) => {
      const { oldTx, newTx, nonce, from } = message.data;
      console.log(`Tx ${oldTx} with nonce ${nonce} from ${from} was sped up, the new hash is ${newTx}`)
    });
    ethereum.on('tx_replacement', (info) => {
      const { oldTx, newTx, nonce, from } = message.data;
      console.log(`Tx ${oldTx} with nonce ${nonce} from ${from} was replaced, the new hash is ${newTx}`)
    });


    console.log(`Transaction hash ${txHash}`)
  })
  .catch((error) => {
    console.error(`Error sending transaction: ${error.code}: ${error.message}`);
  });

```
